@@ -19,7 +19,7 @@
  *
  * Inspired by nsyslog, originally written by Darren Reed.
  *
- * $Id: sources.c,v 1.37.4.7 2005/05/25 08:29:29 bazsi Exp $
+ * $Id: sources.c,v 1.37.4.9 2006/03/13 23:30:26 bazsi Exp $
  *
  ***************************************************************************/
 
@@ -86,79 +86,86 @@ do_read_line(struct read_handler **h,
 	int n;
 	char sabuf[256];
 	socklen_t salen = sizeof(sabuf);
+	int fetch_count = 0;
+	
+	while (fetch_count < 30) {
 
-	if (!closure->dgram) {
-		if (closure->pad_size)
-			n = A_READ(read, LIBOL_MIN(closure->max_log_line, closure->pad_size), closure->buffer + closure->pos);
-		else
-			n = A_READ(read, closure->max_log_line - closure->pos, closure->buffer + closure->pos);
-		salen = 0;
-	}
-	else
-		n = A_RECV(read, closure->max_log_line - closure->pos, closure->buffer + closure->pos, (abstract_addr *) &sabuf, &salen);
-
-	switch(n) {
-	case 0:
-		return ST_OK | ST_GOON;
-	case A_FAIL:
-		/* Fall through */
-		return ST_FAIL | ST_CLOSE;
-	case A_EOF:
-		return ST_OK | ST_CLOSE;
-	}
-	closure->pos += n;
-
-	eol = memchr(closure->buffer, '\0', closure->pos);
-	if (eol == NULL)
-		eol = memchr(closure->buffer, '\n', closure->pos);
-	if (closure->pad_size && eol) {
-		do_handle_line(closure, eol - closure->buffer, closure->buffer, salen ? (abstract_addr *) &sabuf : NULL, salen);
-		closure->pos = 0;
-		return ST_OK | ST_GOON;
-	}
-	if (closure->dgram || (!eol && closure->pos == closure->max_log_line)) {
-		/* we don't have a terminating nl nor \0, and our buffer is
-		   full or we are a datagram receiver, when the message is in
-		   its own packet.
-		 */
-                if (!eol && closure->pos == closure->max_log_line) {
-                        werror("Message length overflow, line is split, log_msg_size=%i\n", closure->max_log_line);
-                }
-		if (closure->dgram) {
-			/* strip one trailing LF or NUL character */
-			if (closure->pos > 0 && 
-			    (closure->buffer[closure->pos - 1] == '\n' || 
-			     closure->buffer[closure->pos - 1] == '\0'))
-				closure->pos--;
+		if (!closure->dgram) {
+			if (closure->pad_size)
+				n = A_READ(read, LIBOL_MIN(closure->max_log_line, closure->pad_size), closure->buffer + closure->pos);
+			else
+				n = A_READ(read, closure->max_log_line - closure->pos, closure->buffer + closure->pos);
+			salen = 0;
 		}
-		do_handle_line(closure, closure->pos, closure->buffer, salen ? (abstract_addr *) &sabuf : NULL, salen);
-		closure->pos = 0;
-		return ST_OK | ST_GOON;
-	}
-	start = closure->buffer;
-	while (eol) {
-		/* eol points at the newline character. end points at the
-		 * character terminating the line, which may be a carriage
-		 * return preceeding the newline. */
-		UINT8 *end = eol;
+		else
+			n = A_RECV(read, closure->max_log_line - closure->pos, closure->buffer + closure->pos, (abstract_addr *) &sabuf, &salen);
 
-		while ((end > start) && (end[-1] == '\r' || end[-1] == '\n' || end[-1] == 0))
-			end--;
+		switch(n) {
+		case 0:
+			return ST_OK | ST_GOON;
+		case A_FAIL:
+			/* Fall through */
+			return ST_FAIL | ST_CLOSE;
+		case A_EOF:
+			return ST_OK | ST_CLOSE;
+		}
+		closure->pos += n;
 
-		length = end - start;
-		
-		if (length)
-			do_handle_line(closure, length, start, salen ? (abstract_addr *) &sabuf : NULL , salen);
-
-		start = eol + 1;
-
-		eol = memchr(start, '\0', &closure->buffer[closure->pos] - start);
+		eol = memchr(closure->buffer, '\0', closure->pos);
 		if (eol == NULL)
-			eol = memchr(start, '\n', &closure->buffer[closure->pos] - start);
+			eol = memchr(closure->buffer, '\n', closure->pos);
+		if (closure->pad_size && eol) {
+			do_handle_line(closure, eol - closure->buffer, closure->buffer, salen ? (abstract_addr *) &sabuf : NULL, salen);
+			closure->pos = 0;
+			goto next_fetch;
+		}
+		if (closure->dgram || (!eol && closure->pos == closure->max_log_line)) {
+			/* we don't have a terminating nl nor \0, and our buffer is
+			   full or we are a datagram receiver, when the message is in
+			   its own packet.
+			 */
+			if (!eol && closure->pos == closure->max_log_line) {
+				werror("Message length overflow, line is split, log_msg_size=%i\n", closure->max_log_line);
+			}
+			if (closure->dgram) {
+				/* strip one trailing LF or NUL character */
+				if (closure->pos > 0 && 
+				    (closure->buffer[closure->pos - 1] == '\n' || 
+				     closure->buffer[closure->pos - 1] == '\0'))
+					closure->pos--;
+			}
+			do_handle_line(closure, closure->pos, closure->buffer, salen ? (abstract_addr *) &sabuf : NULL, salen);
+			closure->pos = 0;
+			goto next_fetch;
+		}
+		start = closure->buffer;
+		while (eol) {
+			/* eol points at the newline character. end points at the
+			 * character terminating the line, which may be a carriage
+			 * return preceeding the newline. */
+			UINT8 *end = eol;
 
+			while ((end > start) && (end[-1] == '\r' || end[-1] == '\n' || end[-1] == 0))
+				end--;
+
+			length = end - start;
+			
+			if (length)
+				do_handle_line(closure, length, start, salen ? (abstract_addr *) &sabuf : NULL , salen);
+
+			start = eol + 1;
+
+			eol = memchr(start, '\0', &closure->buffer[closure->pos] - start);
+			if (eol == NULL)
+				eol = memchr(start, '\n', &closure->buffer[closure->pos] - start);
+
+		}
+		memmove(closure->buffer, start, &closure->buffer[closure->pos] - start);
+		closure->pos = &closure->buffer[closure->pos] - start;
+		
+	next_fetch:
+		fetch_count++;
 	}
-	memmove(closure->buffer, start, &closure->buffer[closure->pos] - start);
-	closure->pos = &closure->buffer[closure->pos] - start;
 	return ST_OK | ST_GOON;
 }
 
